/// <reference types="node" />
import * as EventEmitter from 'events';
import { ConsumerGroup, KafkaClient, Producer } from 'kafka-node';
import { ConfService, DataRequest } from './types';
import Util from './util';
/**
 *  Clase para las conexiones y publicaciones sobre
 *  la cola de mensajes.
 */
export default abstract class Brokers extends Util {
    /**
     * Cliente a kafka
     * @protected
     */
    protected client: KafkaClient;
    /**
     * Producer kafka
     * @protected
     */
    protected producer: Producer;
    /**
     * Consumer kafka
     * @protected
     */
    protected consumer: ConsumerGroup;
    /**
     * Manejador de eventos
     * @protected
     */
    protected event: EventEmitter;
    /**
     * Manejador de logs
     * @protected
     */
    protected logs: any;
    /**
     * Configuracion del servicio
     * @protected
     */
    protected conf: ConfService;
    /**
     * Topico del servicio
     * @protected
     */
    protected Topic: string;
    /**
     * Servidores de kafka
     * @protected
     */
    protected kafkaHost: string[];
    /**
     * Hosts de los brokers
     * @protected
     */
    protected Hosts: string[];
    /**
     * Conexion a nats
     * @protected
     */
    protected nc: any;
    /**
     * Variable para revisar si NATS se encuentra conectado o desconectado
     * @protected
     */
    private natsIsConnected;
    /**
     * variable que almacena la direccion
     * actual del archivo que se ejecuta.
     * @private
     */
    private cwd;
    /**
     * Instancia del generador de hilos
     * @private
     */
    private thread;
    /**
     * Hilo en reserva para aumental la velocidad
     * de respuesta.
     * @private
     */
    private threadRes;
    /**
     * Indicador si el hilo en reserva se encuentra activo.
     * @private
     */
    private threadResInit;
    /**
     * Instancia de la conexion a NATS
     * @private
     */
    private connNats;
    /**
     * PID de la conexion a la subscripcion en
     * NATS
     * @private
     */
    private pidSub;
    /**
     * Numero de hilos registrados actualmente
     * @private
     */
    private threadTotal;
    /**
     * Maximo numero de hilos que se soportan
     * @private
     */
    private maxThread;
    /**
     * Indicador para conocer si el consumer se encuentra en pause.
     * @private
     */
    private isPauseConsumer;
    /**
     * Indicador del soporte de hilos
     * @private
     */
    private threadSupport;
    /**
     * Contructor de la clase.
     * @param kafkaHost Servidores Kafka
     * @param Topic Topico de escucha
     */
    constructor(Conf: ConfService);
    /**
     * Metodo para validar conexion al broker
     */
    validateConnection(): boolean | Promise<boolean>;
    /**
     * Metodo para realizar el envio de datos a la cola de mensajes
     * @param data Datos a enviar
     * @param topic Topico de publicacion
     * @param key Clave unica
     */
    send(dataMessage: string, topic: string, key?: string): void;
    /**
     * Metodo que procesa la peticion de mensajes.
     * @param data Datos de la peticion.
     */
    abstract message(data: DataRequest): Promise<any>;
    /**
     * Metodo para realizar la publicacion en la cola de mensajes
     * @param data Datos que se quieren publicar
     * @param errorCode Codigo de error
     */
    abstract response(data: DataRequest, errorCode: string | number): void;
    /**
     * Metodo que se ejecuta al ocurrir un excepcion
     * @param topic Topico que se escucha
     * @param data Datos proporcionados
     * @param error Error que se genero
     */
    abstract exception(topic: string, data: string, error: Error): void;
    /**
     * Metodo para realizar la conexion Nats
     * @private
     */
    private connecNats;
    /**
     * Metodo para realizar la conexion al broker de kafka
     * @private
     */
    private connectKafka;
    /**
     * Handler para la escueha de mensajes de NATS
     * @param msg Mensaje de entrada
     * @param reply Id del que solicita el dato
     * @param subject Topico
     */
    private handlerNats;
    /**
     * Metodo para crear una instancia del servicio para
     * el procesamiento del mensaje.
     * @param data Datos que se envian a la instancia
     */
    private processMessage;
    /**
     * Metodo para pausar la llegada de mensajes
     * desde el broker
     * @private
     */
    private pause;
    /**
     * Metodo para activar la escucha de mensajes
     * desde el broker.
     * @private
     */
    private resume;
}
